//********************************************************************************************************************************
//*                                                                                                                              *
//*     3delite's Helper functions for Windows 1.5  3delite 2006-2022                                                           *
//*                                                                                                                              *
//* Win32, Win64 compatible.                                                                                                     *
//*                                                                                                                              *
//* 3delite's Helper functions is free (a mention in the credits would be cool if you use it!).                                  *
//*                                                                                                                              *
//*     https://www.3delite.hu/                                                                                                  *
//*                                                                                                                              *
//*     3delite@3delite.hu                                                                                                       *
//*                                                                                                                              *
//********************************************************************************************************************************

unit Helper3deliteWin;

interface

Uses
    SysUtils,
    Windows;

type
    TMouseButtonType = (mbtLeft, mbtMiddle, mbtRight);

const
    MOUSE_BTN_VKEYS: Array [TMouseButtonType] of Integer = (VK_LBUTTON, VK_MBUTTON, VK_RBUTTON);

    function PrivateExtractIcons(lpszFile: LPCTSTR; nIconIndex, cxIcon, cyIcon: Integer; phIcon : HICON; piconid, nIcons, flags: UINT): UINT; stdcall;
    function FileTime2DateTime(FileTime: TFileTime): TDateTime;
    function RenameDir(DirFrom, DirTo: string; Silent: Boolean): Boolean;
    function FileDelete(FileName: String; Silent: Boolean): Boolean;
    function ExecAndWaitForIt(const s: string): Cardinal;
    function ExecAndWaitForItHidden(const s: string; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
    function ExecAndWaitForItHiddenStd(const s: string; const Params: string; LogFileName: String; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
    function ExecWithParamsAndWaitForItHidden(const s: string; const Params: string; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
    function GetPathDiskSize(Path: String): Int64;
    function GetPathFreeSpace(Path: String; var FreeSpace: Int64): Boolean;
    function HTTPDecode(const AStr: AnsiString): AnsiString;
    function HTTPEncode(const AStr: AnsiString): AnsiString;
    function RuningOn64BitOS: Boolean;
    procedure OpenGoogleSearch(Handle: HWND; SearchString: String);
    procedure OpenYoutubeSearch(Handle: HWND; SearchString: String);
    function GetFileVersionStr(ExeFilename: String): String;
    function GetFileVersionStrEx(ExeFilename: String; Silent: Boolean): String; overload;
    function GetFileDescriptionStr(ExeFilename: String): String;
    function GetFileAuthorStr(ExeFilename: String): String;
    function GetFileCopyrightStr(ExeFilename: String): String;
    function DriveSerialNumber(DriveID:Byte): string;
    procedure ProcessMessages;
    function FileExistsOnVideoDVD(const FileName: String): Boolean;
    function WindowsXPThemeActive: Boolean;
    function CtrlDown: Boolean;
    function ShiftDown: Boolean;
    function AltDown: Boolean;
    function EjectTray(Drive: Char): Boolean;
    function InsertTray(Drive: Char): Boolean;
    function GetProcMemUsage(Proc: Cardinal): Int64;
    function GetTimeZoneBias: TDateTime;
    function BlockSize(Data: Pointer): Integer; register;
    function ExecProcessHandle(ProgramName: String): THandle;
    function SetDelimitedText(const Value: string; Delimiter, QuoteChar: Char): String;
    function GetWinSystemPath: String;
    function GetWinTempPath: String;
    function GetWinTempFileName(Beggining, Extension: String): String; stdcall; export;
    function GetLoginName: String;
    function GetComputerNameStr: String;
    function GetVolumeLabel(Drive: String): string;
    function GetCPUCount: Integer;
    function IsMouseBtnSwapped: Boolean;
    function IsMouseBtnDown(AMouseBtn: TMouseButtonType): Boolean;
    function FileExistsEx(const FileName: String): Boolean;
    function LocalDateTimeToUTC(D: TDateTime): TDateTime;
    function UTCToLocalDateTime(D: TDateTime): TDateTime;
    procedure UpdateFileDirectoryEntry(FileName: String);
    procedure SetFileCreationTime(const FileName: string; const DateTime: TDateTime);
    function GetProcessRAMWorkingSetSize: Int64;

const
  VK_ENTER    = 13;
  VK_ESC      = 27;

  LVS_EX_LABELTIP = $00004000;

{ from winioctl.h }

  FILE_DEVICE_DISK  = $00000007;
  FILE_DEVICE_FILE_SYSTEM = $00000009;
  METHOD_BUFFERED   = 0;
  IOCTL_DISK_BASE   = FILE_DEVICE_DISK;
  FILE_READ_ACCESS  = 1; // file & pipe
  FILE_ANY_ACCESS   = 0;
  FILE_DEVICE_MASS_STORAGE = $2D;
  FILE_WRITE_ACCESS = 2;

  IOCTL_DISK_EJECT_MEDIA = (IOCTL_DISK_BASE shl 16) OR ($0202 shl 14) OR (METHOD_BUFFERED shl 2) OR FILE_READ_ACCESS;
  FSCTL_LOCK_VOLUME = (FILE_DEVICE_FILE_SYSTEM shl 16) OR (6 shl 14) OR (METHOD_BUFFERED shl 2) OR FILE_ANY_ACCESS;
  FSCTL_UNLOCK_VOLUME = (FILE_DEVICE_FILE_SYSTEM shl 16) OR (7 shl 14) OR (METHOD_BUFFERED shl 2) OR FILE_ANY_ACCESS;

  IOCTL_STORAGE_MEDIA_REMOVAL : DWORD = ($2D shl 16) OR (1 shl 14) OR ($201 shl 2);
  //IOCTL_STORAGE_EJECT_MEDIA   : DWORD = ($2D shl 16) OR (1 shl 14) OR ($202 shl 2);
  //FSCTL_LOCK_VOLUME           : DWORD = ( $9 shl 16) OR (0 shl 14) OR (  $6 shl 2);
  //FSCTL_UNLOCK_VOLUME         : DWORD = ( $9 shl 16) OR (0 shl 14) OR (  $7 shl 2);

  IOCTL_STORAGE_EJECT_MEDIA = (FILE_DEVICE_MASS_STORAGE shl 16) OR (FILE_READ_ACCESS shl 14) OR ($202 shl 2) OR (METHOD_BUFFERED);
  IOCTL_STORAGE_LOAD_MEDIA = (FILE_DEVICE_MASS_STORAGE shl 16) OR (FILE_READ_ACCESS shl 14) OR ($203 shl 2) OR (METHOD_BUFFERED);

const
  BELOW_NORMAL_PRIORITY_CLASS     = $00004000;
  {$EXTERNALSYM BELOW_NORMAL_PRIORITY_CLASS}
  ABOVE_NORMAL_PRIORITY_CLASS     = $00008000;
  {$EXTERNALSYM ABOVE_NORMAL_PRIORITY_CLASS}

type
    STORAGE_PROPERTY_QUERY = packed record
        PropertyId: DWORD;
        QueryType: DWORD;
        AdditionalParameters: Array[0..3] of Byte;
    end;

type
    STORAGE_BUS_TYPE = DWORD;

type
    STORAGE_DEVICE_DESCRIPTOR = packed record
        Version: DWORD;
        Size: DWORD;
        DeviceType: Byte;
        DeviceTypeModifier: Byte;
        RemovableMedia: Bool;
        CommandQueueing: Bool;
        VendorIdOffset: DWORD;
        ProductIdOffset: DWORD;
        ProductRevisionOffset: DWORD;
        SerialNumberOffset: DWORD;
        BusType: STORAGE_BUS_TYPE;
        RawPropertiesLength: DWORD;
        RawDeviceProperties: Array[0..1] of Byte;
    end;

const
    MAX_IDE_DRIVES = 16;

type
    _STORAGE_PROPERTY_ID = (StorageDeviceProperty, StorageAdapterProperty);

type
    _STORAGE_QUERY_TYPE = (PropertyStandardQuery, PropertyExistsQuery, PropertyMaskQuery, PropertyQueryMaxDefined);

const
    IOCTL_STORAGE_QUERY_PROPERTY = 2954240;

implementation

Uses
    AnsiStrings,
    PSApi,
    ShellApi,
    Registry,
    SpecialFolders;

    function PrivateExtractIcons; external user32 name 'PrivateExtractIcons';


function FileTime2DateTime(FileTime: TFileTime): TDateTime;
var
  LocalFileTime: TFileTime;
  SystemTime: TSystemTime;
begin
    FileTimeToLocalFileTime(FileTime, LocalFileTime);
    FileTimeToSystemTime(LocalFileTime, SystemTime);
    Result := SystemTimeToDateTime(SystemTime);
end;

function RenameDir(DirFrom, DirTo: string; Silent: Boolean): Boolean;
var
  ShellInfo: TSHFileOpStruct;
begin
    if UpperCase(DirFrom) = UpperCase(DirTo) then begin
        Result := True;
        Exit;
    end;
    with ShellInfo do begin
        Wnd    := 0;
        wFunc  := FO_RENAME;
        pFrom  := PChar(DirFrom + #0);
        pTo    := PChar(DirTo + #0);
        if Silent    //OR FOF_SILENT
            then fFlags := FOF_ALLOWUNDO OR FOF_NOCONFIRMATION
            else fFlags := FOF_ALLOWUNDO;
    end;
    if SHFileOperation(ShellInfo) = 0
        then Result := True
        else Result := False;
end;

function FileDelete(FileName: String; Silent: Boolean): Boolean;
var
  DelStruct: TSHFileOpStruct;
  //ReturnValue: Integer;
begin
    //* Directory
    if DirectoryExists(FileName) then begin
        ZeroMemory(@DelStruct, SizeOf(DelStruct));
        with DelStruct do begin
            Wnd    := 0;
            wFunc  := FO_DELETE;
            pFrom  := PWideChar(FileName + #0);
            fFlags := FOF_ALLOWUNDO
                OR FOF_NOCONFIRMMKDIR;
            if Silent
                then fFlags := fFlags OR FOF_SILENT;
        end;
        {ReturnValue :=} SHFileOperation(DelStruct);
        if DirectoryExists(FileName)
            then Result := False
            else Result := True;
        Exit;
    end;
    //* File
    ZeroMemory(@DelStruct, SizeOf(DelStruct));
    with DelStruct do begin
        Wnd    := 0;
        wFunc  := FO_DELETE;
        pFrom  := PWideChar(FileName + #0);
        fFlags := FOF_ALLOWUNDO
            OR FOF_NOCONFIRMMKDIR;
        if Silent
            then fFlags := fFlags OR FOF_SILENT;
    end;
    {ReturnValue :=} SHFileOperation(DelStruct);
    if FileExists(FileName)
        then Result := False
        else Result := True;
end;

function ExecAndWaitForIt(const s: string): Cardinal;
var
    StartUpInfo: TStartUpInfo;
    ProcessInfo: TProcessInformation;
    Command: array[0..255] of Char;
begin
    FillChar(StartUpInfo, SizeOf(StartUpInfo), #0);
    StartUpInfo.cb := sizeof(Startupinfo);
    StartUpInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartUpInfo.wShowWindow := sw_Show;
    StrPCopy(Command,s);
    if CreateProcess(nil,
        Command,
        nil,
        nil,
        False,
        CREATE_NEW_CONSOLE OR NORMAL_PRIORITY_CLASS,
        nil,
        nil,
        StartUpInfo,
        ProcessInfo)
    then begin
        WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
        GetExitCodeProcess(ProcessInfo.hProcess, Result);
        CloseHandle(ProcessInfo.hThread);
        CloseHandle(ProcessInfo.hProcess);
    end;
end;

{
function ExecAndWaitForItHidden(const s: string): Cardinal;
var
    StartUpInfo: TStartUpInfo;
    ProcessInfo: TProcessInformation;
    Command: array[0..255] of Char;
begin
    FillChar(StartUpInfo, SizeOf(StartUpInfo), #0);
    StartUpInfo.cb := sizeof(Startupinfo);
    StartUpInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartUpInfo.wShowWindow := SW_HIDE;
    StrPCopy(Command,s);
    if CreateProcess(nil,
        Command,
        nil,
        nil,
        False,
        CREATE_NEW_CONSOLE OR NORMAL_PRIORITY_CLASS,
        nil,
        nil,
        StartUpInfo,
        ProcessInfo)
    then begin
        WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
        GetExitCodeProcess(ProcessInfo.hProcess, Result);
        CloseHandle(ProcessInfo.hThread);
        CloseHandle(ProcessInfo.hProcess);
    end;
end;
}

function ExecAndWaitForItHidden(const s: string; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
var
    StartUpInfo: TStartUpInfo;
    ProcessInfo: TProcessInformation;
    Command: array[0..255 * 100] of Char;
begin
    FillChar(StartUpInfo, SizeOf(StartUpInfo), #0);
    StartUpInfo.cb := sizeof(Startupinfo);
    StartUpInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartUpInfo.wShowWindow := SW_HIDE;
    StrPCopy(Command, s);
    if CreateProcess(nil,
        Command,
        nil,
        nil,
        False,
        CREATE_NEW_CONSOLE OR Priority,
        nil,
        nil,
        StartUpInfo,
        ProcessInfo)
    then begin
        WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
        GetExitCodeProcess(ProcessInfo.hProcess, Result);
        CloseHandle(ProcessInfo.hThread);
        CloseHandle(ProcessInfo.hProcess);
    end;
end;

function ExecAndWaitForItHiddenStd(const s: string; const Params: string; LogFileName: String; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
var
    StartUpInfo: TStartUpInfo;
    ProcessInfo: TProcessInformation;
    //Command: array[0..255] of Char;
    sa: SECURITY_ATTRIBUTES;
    LogHandle: THandle;
begin
    FillChar(StartUpInfo, SizeOf(StartUpInfo), #0);
    StartUpInfo.cb := sizeof(Startupinfo);
    StartUpInfo.dwFlags := STARTF_USESHOWWINDOW OR STARTF_USESTDHANDLES;

    sa.nLength := sizeof(sa);
    sa.lpSecurityDescriptor := nil;
    sa.bInheritHandle := TRUE;

    LogHandle := CreateFile(PChar(LogFileName),
        FILE_APPEND_DATA OR FILE_WRITE_DATA,
        FILE_SHARE_WRITE OR FILE_SHARE_READ,
        @sa,
        CREATE_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        0);

    StartUpInfo.hStdInput := 0;
    StartUpInfo.hStdError := LogHandle;
    StartUpInfo.hStdOutput := LogHandle;

    StartUpInfo.wShowWindow := SW_HIDE;
    //StrPCopy(Command,s);
    if CreateProcess(//nil,
        PChar(s),
        PChar(Params),
        nil,
        nil,
        False,
        CREATE_NEW_CONSOLE OR Priority,
        nil,
        nil,
        StartUpInfo,
        ProcessInfo)
    then begin
        WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
        GetExitCodeProcess(ProcessInfo.hProcess, Result);
        CloseHandle(ProcessInfo.hThread);
        CloseHandle(ProcessInfo.hProcess);

        CloseHandle(LogHandle);

    end;
end;

function ExecWithParamsAndWaitForItHidden(const s: string; const Params: string; Priority: Cardinal = NORMAL_PRIORITY_CLASS): Cardinal;
var
    StartUpInfo: TStartUpInfo;
    ProcessInfo: TProcessInformation;
    //Command: array[0..255] of Char;
begin
    FillChar(StartUpInfo, SizeOf(StartUpInfo), #0);
    StartUpInfo.cb := sizeof(Startupinfo);
    StartUpInfo.dwFlags := STARTF_USESHOWWINDOW;
    StartUpInfo.wShowWindow := SW_HIDE;
    //StrPCopy(Command,s);
    if CreateProcess(
        PChar(s),
        PChar(Params),
        nil,
        nil,
        False,
        CREATE_NEW_CONSOLE OR Priority,
        nil,
        nil,
        StartUpInfo,
        ProcessInfo)
    then begin
        WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
        GetExitCodeProcess(ProcessInfo.hProcess, Result);
        CloseHandle(ProcessInfo.hThread);
        CloseHandle(ProcessInfo.hProcess);
    end;
end;

function GetPathDiskSize(Path: String): Int64;
var
  Letter: String;
begin
    try
        Result := 0;
        if NOT DirectoryExists(Path) then begin
            Exit;
        end;
        Letter := ExtractFileDrive(Path);
        if Length(Letter) > 0 then begin
            Result := DiskSize(Ord(Letter[1]) - $40);
        end;
        if Result < 0
            then Result := 0;
    except
        Result := 0;
    end;
end;

function GetPathFreeSpace(Path: String; var FreeSpace: Int64): Boolean;
var
    Letter: String;
begin
    Result := False;
    FreeSpace := 0;
    try
        if NOT DirectoryExists(Path) then begin
            Exit;
        end;
        Letter := ExtractFileDrive(Path);
        if Length(Letter) > 0 then begin
            FreeSpace := DiskFree(Ord(Letter[1]) - $40);
        end;
        if FreeSpace < 0 then begin
            Result := False;
        end else begin
            Result := True;
        end;
    except
        //*
    end;
end;

function GetFileVersionStr(ExeFilename: String): String;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  S: string;
  n, Len, i: DWORD;
  Buf: PChar;
  Value: PChar;
begin
    Result := 'Unknown version';
    S := ExeFilename;
    n := GetFileVersionInfoSize(PChar(S), n);
    if n > 0 then begin
        Buf := AllocMem(n);
        GetFileVersionInfo(PChar(S), 0, n, Buf);
        for i := 1 to InfoNum
            do if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len)
                then if InfoStr[i] = 'FileVersion' then begin
                    Result := Value;
                    Break;
                end;
        FreeMem(Buf, n);
    end;
end;

function GetFileVersionStrEx(ExeFilename: String; Silent: Boolean): String; overload;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  S: string;
  n, Len, i: DWORD;
  Buf: PChar;
  Value: PChar;
begin
    if NOT Silent
        then Result := 'unknown version'
        else Result := '';
    S := ExeFilename;
    n := GetFileVersionInfoSize(PChar(S), n);
    if n > 0 then begin
        Buf := AllocMem(n);
        GetFileVersionInfo(PChar(S), 0, n, Buf);
        for i := 1 to InfoNum
            do if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len)
                then if InfoStr[i] = 'FileVersion' then begin
                    Result := Value;
                    Break;
                end;
        FreeMem(Buf, n);
    end;
end;

function GetFileDescriptionStr(ExeFilename: String): String;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  S: string;
  n, Len, i: DWORD;
  Buf: PChar;
  Value: PChar;
begin
    Result := '';
    S := ExeFilename;
    n := GetFileVersionInfoSize(PChar(S), n);
    if n > 0 then begin
        Buf := AllocMem(n);
        GetFileVersionInfo(PChar(S), 0, n, Buf);
        for i := 1 to InfoNum
            do if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len)
                then if InfoStr[i] = 'FileDescription' then begin
                    Result := Value;
                    Break;
                end;
        FreeMem(Buf, n);
    end;
    //if Trim(Result) = ''
    //    then Result := ExtractFileName(ExeFilename);
end;

function GetFileAuthorStr(ExeFilename: String): String;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  S: string;
  n, Len, i: DWORD;
  Buf: PChar;
  Value: PChar;
begin
    Result := '';
    S := ExeFilename;
    n := GetFileVersionInfoSize(PChar(S), n);
    if n > 0 then begin
        Buf := AllocMem(n);
        GetFileVersionInfo(PChar(S), 0, n, Buf);
        for i := 1 to InfoNum
            do if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len)
                then if InfoStr[i] = 'CompanyName' then begin
                    Result := Value;
                    Break;
                end;
        FreeMem(Buf, n);
    end;
    if Trim(Result) = ''
        then Result := '';
end;

function GetFileCopyrightStr(ExeFilename: String): String;
const
  InfoNum = 10;
  InfoStr: array[1..InfoNum] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName', 'LegalCopyright', 'LegalTradeMarks', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments');
var
  S: string;
  n, Len, i: DWORD;
  Buf: PChar;
  Value: PChar;
begin
    Result := '';
    S := ExeFilename;
    n := GetFileVersionInfoSize(PChar(S), n);
    if n > 0 then begin
        Buf := AllocMem(n);
        GetFileVersionInfo(PChar(S), 0, n, Buf);
        for i := 1 to InfoNum
            do if VerQueryValue(Buf, PChar('StringFileInfo\040904E4\' + InfoStr[i]), Pointer(Value), Len)
                then if InfoStr[i] = 'LegalCopyright' then begin
                    Result := Value;
                    Break;
                end;
        FreeMem(Buf, n);
    end;
    if Trim(Result) = ''
        then Result := '';
end;

function DriveSerialNumber(DriveID:Byte): string;
  var
    hPhysicalDriveIOCTL    :THandle;
    query                  :STORAGE_PROPERTY_QUERY;
    buffer                 :array [0..10000] of byte;
    descrip                :STORAGE_DEVICE_DESCRIPTOR absolute buffer;
    cbBytesReturned        :DWORD;
    serial                 :PChar;
    s                      :string;
    i                      :Integer;
    ModelNumber            :string;
    SerialNumber           :string;
  begin
    Result:=EmptyStr;
    if DriveID>=MAX_IDE_DRIVES then
      begin
        SetLastError(ERROR_INVALID_DRIVE);
        Exit;
      end;
    if Win32Platform<>VER_PLATFORM_WIN32_NT then
      begin
        SetLastError(ERROR_APP_WRONG_OS);
        Exit;
      end;
    hPhysicalDriveIOCTL:=CreateFile(PChar('\\.\PhysicalDrive'+IntToStr(DriveID)),
                                    0,
                                    FILE_SHARE_READ or FILE_SHARE_WRITE,
                                    NIL,
                                    OPEN_EXISTING,
                                    0,
                                    0);
    if hPhysicalDriveIOCTL=INVALID_HANDLE_VALUE then Exit;
    try
      FillChar(query,SizeOf(query),0);
      query.PropertyID:=DWORD(StorageDeviceProperty);
      query.QueryType:=DWORD(PropertyStandardQuery);
      FillChar(buffer,SizeOf(buffer),0);
      cbBytesReturned:=0;
      if not DeviceIoControl(hPhysicalDriveIOCTL,
                             IOCTL_STORAGE_QUERY_PROPERTY,
                             @query,
                             SizeOf(query),
                             @buffer,
                             SizeOf(buffer),
                             cbBytesReturned,
                             NIL) then Exit;
      SerialNumber:=EmptyStr;
      if descrip.SerialNumberOffset<>0 then
        begin
          serial:=@buffer[descrip.SerialNumberOffset];
          i:=0;
          while Cardinal(i+3)<StrLen(serial) do
            begin
              SetString(s,PChar(Integer(serial)+i+2),2);
              Insert('$',s,1);
              SerialNumber:=SerialNumber+chr(StrToInt(s));
              SetString(s,PChar(Integer(serial)+i),2);
              Insert('$',s,1);
              SerialNumber:=SerialNumber+chr(StrToInt(s));
              Inc(i,4);
            end;
        end;
      ModelNumber:=EmptyStr;
      if descrip.ProductIDOffset<>0 then
        begin
          serial:=@buffer[descrip.ProductIDOffset];
          ModelNumber:=string(serial);
        end;
      Result:=Format('Model Number=%s     Serial Number=%s',[Trim(ModelNumber),Trim(SerialNumber)]);
    finally
      CloseHandle(hPhysicalDriveIOCTL);
    end;
    SetLastError(NO_ERROR);
end;

procedure ProcessMessages;
var
  Msg : tMsg;
begin
    while PeekMessage(Msg,0,0,0,PM_REMOVE) do begin
        TranslateMessage(Msg);
        DispatchMessage(Msg);
    end;
end;

function FileExistsOnVideoDVD(const FileName: String): Boolean;
var
  Handle: THandle;
  FindData: TWin32FindData;
begin
    Result := False;
    Handle := FindFirstFile(PChar(FileName), FindData);
    if Handle <> INVALID_HANDLE_VALUE then begin
        Windows.FindClose(Handle);
        if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0
            then Result := True;
    end;
end;

function WindowsXPThemeActive: Boolean;
var
  Reg: TRegistry;
  ThemeActive: String;
begin
    Result := False;
    Reg := TRegistry.Create;
    try
        try
            Reg.RootKey := HKEY_CURRENT_USER;
            Reg.OpenKey('Software\Microsoft\Windows\CurrentVersion\ThemeManager\', False);
            if Reg.ValueExists('ThemeActive')
                then ThemeActive := Reg.ReadString('ThemeActive')
                else ThemeActive := '0';
            Result := ThemeActive <> '0';
        except
            //*
        end;
    finally
        Reg.CloseKey;
        FreeAndNil(Reg);
    end;
end;

function CtrlDown: Boolean;
var
  State: TKeyboardState;
begin
    GetKeyboardState(State);
    Result := ((State[VK_CONTROL] AND 128) <> 0);
end;

function ShiftDown: Boolean;
var
  State: TKeyboardState;
begin
    GetKeyboardState(State);
    Result := ((State[VK_SHIFT] AND 128) <> 0);
end;

function AltDown: Boolean;
var
  State: TKeyboardState;
begin
    GetKeyboardState(State);
    Result := ((State[VK_MENU] AND 128) <> 0);
end;

function EjectTray(Drive: Char): Boolean;
var
  hDevice: THandle;
  cb: DWORD;
  lpFileName: PChar;
begin
    Result := False;
    lpFileName := PChar('\\\\\\\\\\\?\\\\\.\\\\\\\\' + UpperCase(Drive) + ':');
    hDevice := CreateFile(lpFileName, GENERIC_READ, FILE_SHARE_READ OR FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
    if hDevice = INVALID_HANDLE_VALUE
        then Exit;
    Result := DeviceIoControl(hDevice, IOCTL_STORAGE_EJECT_MEDIA, nil, 0, nil, 0, cb, nil);
    CloseHandle(hDevice);
end;

function InsertTray(Drive: Char): Boolean;
var
  hDevice: THandle;
  cb: DWORD;
  lpFileName: PChar;
begin
    Result := False;
    lpFileName := PChar('\\\\\\\\\\\?\\\\\.\\\\\\\\' + UpperCase(Drive) + ':');
    hDevice := CreateFile(lpFileName, GENERIC_READ, FILE_SHARE_READ OR FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
    if hDevice = INVALID_HANDLE_VALUE
        then Exit;
    Result := DeviceIoControl(hDevice, IOCTL_STORAGE_LOAD_MEDIA , nil, 0, nil, 0, cb, nil) = False;
    CloseHandle(hDevice);
end;

function GetProcMemUsage(Proc: Cardinal): Int64;
var
  PMEMC: _PROCESS_MEMORY_COUNTERS;
begin
    Result := 0;
    PMEMC.cb := SizeOf(_PROCESS_MEMORY_COUNTERS);
    if GetProcessMemoryInfo(Proc, @PMEMC, SizeOf(_PROCESS_MEMORY_COUNTERS))
        then Result:= PMEMC.WorkingSetSize;
end;

function GetTimeZoneBias: TDateTime;
var
  TZI: TTimeZoneInformation;
begin
    Result := 0;
    case GetTimeZoneInformation(TZI) of
        TIME_ZONE_ID_STANDARD: Result := (TZI.Bias + TZI.StandardBias) / 1440;
        TIME_ZONE_ID_DAYLIGHT: Result := (TZI.Bias + TZI.DaylightBias) / 1440;
        TIME_ZONE_ID_UNKNOWN: Result := TZI.Bias / 1440;
    else
        //RaiseLastWin32Error;
    end;
end;

function BlockSize(Data: Pointer): Integer; register;
asm
    MOV     EAX, [EAX-04h]
    AND     EAX, 7FFFFFFCh
    SUB     EAX, 04h
end;

function ExecProcessHandle(ProgramName: String): THandle;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
begin
    FillChar(SI, SizeOf(TStartupInfo), 0);
    FillChar(PI, SizeOf(TProcessInformation), 0);
    SI.cb := SizeOf(TStartupInfo);
    CreateProcess(nil, PChar(ProgramName), nil, nil, False, NORMAL_PRIORITY_CLASS, nil, nil, SI, PI);
    Result := PI.hProcess;
end;

function SetDelimitedText(const Value: string; Delimiter, QuoteChar: Char): String;
var
  P, P1: PChar;
  S: string;
begin
    Result := '';
    try
        P := PChar(Value);



        while CharInSet(P^, [#1..' '])
            do P := CharNext(P);
        while P^ <> #0 do begin
            if P^ = QuoteChar
                then S := AnsiExtractQuotedStr(P, QuoteChar)
                else begin
                    P1 := P;
                    while (P^ <> '') AND (P^ <> Delimiter)
                        do P := CharNext(P);
                    SetString(S, P1, P - P1);
                end;
            Result := Result + (#13#10) + S;
            //Add(S);
            while CharInSet(P^, [#1..' '])
                do P := CharNext(P);
            if P^ = Delimiter then begin
                P1 := P;
                if CharNext(P1)^ = #0
                    then Result := Result + #1010 + '';
                repeat
                    P := CharNext(P);
                until not CharInSet(P^, [#1..' ']);
            end;
        end;
    finally
        try
            Result := Copy(Result, 3, Length(Result));
        except
            //*
        end;
    end;
end;

function GetWinSystemPath: String;
var
    TmpDir: array [0..MAX_PATH-1] of Char;
begin
    GetSystemDirectory(TmpDir, MAX_PATH);
    Result := IncludeTrailingPathDelimiter(TmpDir);
end;

function GetWinTempPath: String;
var
    TmpDir: array [0..MAX_PATH-1] of Char;
begin
    GetTempPath(MAX_PATH, TmpDir);
    Result := IncludeTrailingPathDelimiter(TmpDir);
end;

function GetWinTempFileName(Beggining, Extension: String): String; stdcall; export;
var
  TmpDir: array [0..MAX_PATH-1] of Char;
begin
    GetTempPath(MAX_PATH, TmpDir);
	Randomize;
    repeat
    	Result := IncludeTrailingPathDelimiter(TmpDir) + Beggining + IntToStr(Random(1000000)) + '.' + Extension;
    until NOT FileExists(Result);
end;

function GetLoginName: String;
var
  Buffer: Array[0..255] of Char;
  Size: dword;
begin
    Size := High(Buffer) + 1;
    if GetUserName(Buffer, Size)
        then Result := Buffer
        else Result := '';
end;

function GetComputerNameStr: String;
var
  Buffer: Array[0..255] of Char;
  Size: dword;
begin
    Size := High(Buffer) + 1;
    if GetComputerName(Buffer, Size)
        then Result := Buffer
        else Result := '';
end;

function HTTPDecode(const AStr: AnsiString): AnsiString;
var
  Sp, Rp, Cp: PAnsiChar;
  S: AnsiString;
begin
  SetLength(Result, Length(AStr));
  Sp := PAnsiChar(AStr);
  Rp := PAnsiChar(Result);
  Cp := Sp;
  try
    while Sp^ <> #0 do
    begin
      case Sp^ of
        //'%20': Rp^ := ' ';
        '%': begin
               // Look for an escaped % (%%) or %<hex> encoded character
               Inc(Sp);
               if Sp^ = '%' then
                 Rp^ := '%'
               else
               begin
                 Cp := Sp;
                 Inc(Sp);
                 if (Cp^ <> #0) and (Sp^ <> #0) then
                 begin
                   S := AnsiChar('$') + Cp^ + Sp^;
                   Rp^ := AnsiChar(StrToInt(string(S)));
                 end
                 else
                   raise Exception.CreateFmt('ErrorDecodingURLText', [Cp - PAnsiChar(AStr)]);
               end;
             end;
      else
        Rp^ := Sp^;
      end;
      Inc(Rp);
      Inc(Sp);
    end;
  except
    on E:EConvertError do
      raise EConvertError.CreateFmt('sInvalidURLEncodedChar',
        [AnsiChar('%') + Cp^ + Sp^, Cp - PAnsiChar(AStr)])
  end;
  SetLength(Result, Rp - PAnsiChar(Result));
end;

function HTTPEncode(const AStr: AnsiString): AnsiString;
// The NoConversion set contains characters as specificed in RFC 1738 and
// should not be modified unless the standard changes.
//* Modified by 3delite to support already encoded characters (do not encode them again)
const
    NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-',
                    '0'..'9','$','!','''','(',')', '/', ':'];
var
    Sp, Rp: PAnsiChar;
begin
    SetLength(Result, Length(AStr) * 3);
    Sp := PAnsiChar(AStr);
    Rp := PAnsiChar(Result);
    while Sp^ <> #0 do begin
        if Sp^ = '%' then begin
            Rp^ := Sp^;
            Inc(Rp);
            Inc(Sp);
            Rp^ := Sp^;
            Inc(Rp);
            Inc(Sp);
            Rp^ := Sp^;
            Inc(Rp);
            Inc(Sp);
        end else begin
            if Sp^ in NoConversion then begin
              Rp^ := Sp^
            end else begin
                AnsiStrings.FormatBuf(Rp^, 3, AnsiString('%%%.2x'), 6, [Ord(Sp^)]);
                Inc(Rp,2);
            end;
            Inc(Rp);
            Inc(Sp);
        end;
    end;
    SetLength(Result, Rp - PAnsiChar(Result));
end;

function RuningOn64BitOS: Boolean;
type
    TIsWow64Process = function(Handle:THandle; var IsWow64 : BOOL) : BOOL; stdcall;
var
    hKernel32: HMODULE;
    IsWow64Process: TIsWow64Process;
    IsWow64: BOOL;
begin
  // we can check if the operating system is 64-bit by checking whether
  // we are running under Wow64 (we are 32-bit code). We must check if this
  // function is implemented before we call it, because some older versions
  // of kernel32.dll (eg. Windows 2000) don't know about it.
  // see http://msdn.microsoft.com/en-us/library/ms684139%28VS.85%29.aspx
    Result := False;
    hKernel32 := LoadLibrary('kernel32.dll');
    if hKernel32 <> 0 then begin
        try
            @IsWow64Process := GetProcAddress(hkernel32, 'IsWow64Process');
            if Assigned(IsWow64Process) then begin
                IsWow64 := False;
            end;
            if (IsWow64Process(GetCurrentProcess, IsWow64)) then begin
                Result := IsWow64;
            end;
        finally
            FreeLibrary(hKernel32);
        end;
    end;
end;

function GetVolumeLabel(Drive: String): string;
type
    VolumeInfo = Array[0..MAX_PATH] of Char;
var
    NotUsed:     DWORD;
    VolumeFlags: DWORD;
    VolumeSerialNumber: DWORD;
    Buf: array [0..MAX_PATH] of Char;
begin
    GetVolumeInformation(PChar(Drive), Buf, SizeOf(VolumeInfo), @VolumeSerialNumber, NotUsed, VolumeFlags, nil, 0);
    SetString(Result, Buf, StrLen(Buf));   { Set return result }
    //Result:=AnsiUpperCase(Result)
end;

function GetCPUCount: Integer;
var
    SystemInfo: TSystemInfo;
begin
    GetSystemInfo(SystemInfo);
    Result := SystemInfo.dwNumberOfProcessors;
end;

procedure OpenGoogleSearch(Handle: HWND; SearchString: String);
var
    URLString: String;
begin
    URLString := SearchString;
    URLString := StringReplace(URLString, '&', '%26', [rfReplaceAll]);
    URLString := StringReplace(URLString, ' ', '+', [rfReplaceAll]);
    URLString := 'http://www.google.com/search?as_q=' + URLString;
    ShellExecute(Handle, 'open', PChar(URLString), nil, nil, SW_SHOWNORMAL);
end;

procedure OpenYoutubeSearch(Handle: HWND; SearchString: String);
var
    URLString: String;
begin
    URLString := SearchString;
    URLString := StringReplace(URLString, '&', '%26', [rfReplaceAll]);
    URLString := StringReplace(URLString, ' ', '+', [rfReplaceAll]);
    URLString := 'https://www.youtube.com/results?search_query=' + URLString;
    ShellExecute(Handle, 'open', PChar(URLString), nil, nil, SW_SHOWNORMAL);
end;

(* This function returns true if the mouse buttons are swapped *)
function IsMouseBtnSwapped: Boolean;
begin
    Result := GetSystemMetrics(SM_SWAPBUTTON) <> 0;
end;

(* This function returns true when the specified mouse button is pressed *)
function IsMouseBtnDown(AMouseBtn: TMouseButtonType): Boolean;
begin
    if IsMouseBtnSwapped then begin
        if AMouseBtn = mbtLeft then begin
            AMouseBtn := mbtRight;
        end;
        if AMouseBtn = mbtRight then begin
            AMouseBtn := mbtLeft;
        end;
    end;
    Result := GetAsyncKeyState(MOUSE_BTN_VKEYS[AMouseBtn]) AND $8000 <> 0;
end;

function FileExistsEx(const FileName: String): Boolean;
begin
    Result := FileExists(FileName);
    if NOT Result
        then Result := FileExistsOnVideoDVD(FileName);
end;

function LocalDateTimeToUTC(D: TDateTime): TDateTime;
begin
    Result := D + GetTimeZoneBias;
end;

function UTCToLocalDateTime(D: TDateTime): TDateTime;
begin
    Result := D - GetTimeZoneBias;
end;

procedure UpdateFileDirectoryEntry(FileName: String);
var
    h: THandle;
begin
    h := CreateFile(
        PChar(FileName),
        0,                  // don't require any access at all
        FILE_SHARE_READ OR FILE_SHARE_WRITE OR FILE_SHARE_DELETE,
        nil,               // lpSecurityAttributes
        OPEN_EXISTING,
        0,                  // dwFlagsAndAttributes
        0);              // hTemplateFile
    if (h <> INVALID_HANDLE_VALUE) then begin
        CloseHandle(h);
    end;
end;

procedure SetFileCreationTime(const FileName: string; const DateTime: TDateTime);
var
    Handle: THandle;
    SystemTime: TSystemTime;
    FileTime: TFileTime;
begin
    Handle := CreateFile(PChar(FileName), FILE_WRITE_ATTRIBUTES, FILE_SHARE_READ or FILE_SHARE_WRITE, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
    if Handle=INVALID_HANDLE_VALUE then begin
        RaiseLastOSError;
    end;
    try
        DateTimeToSystemTime(DateTime, SystemTime);
        if not SystemTimeToFileTime(SystemTime, FileTime) then begin
            RaiseLastOSError;
        end;
        if not SetFileTime(Handle, @FileTime, nil, nil) then begin
            RaiseLastOSError;
        end;
    finally
        CloseHandle(Handle);
    end;
end;

function GetProcessRAMWorkingSetSize: Int64;
var
    PMEMC: _PROCESS_MEMORY_COUNTERS;
begin
    Result := 0;
    PMEMC.cb := SizeOf(_PROCESS_MEMORY_COUNTERS);
    if GetProcessMemoryInfo(GetCurrentProcess, @PMEMC, SizeOf(_PROCESS_MEMORY_COUNTERS)) then begin
        Result := PMEMC.WorkingSetSize;
    end;
end;

end.
